<?php

namespace App\Http\Controllers\V1\Insignia;

use App\Http\Controllers\Controller;
use App\Models\Insignia;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class InsigniaController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $insignias = Insignia::with('files')->get();
            return $insignias;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las insignias"]]], 400);
        }
    }

    /**
     * Obtiene la informacion de un insignia
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $insigniaQuery = Insignia::with(['files', 'files_premio'])
                ->where('id', $id)->firstOrFail();
            return $insigniaQuery;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar la insignia"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only(
            'nombre',
            'descripcion_premio',
            'objetivo',
            'user_id',
            'file_id',
            'file_id_premio',
        );
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion_premio' => 'required|max:600|string',
            'objetivo' => 'required|max:600|string',
            'user_id' => 'required|integer|exists:users,id',
            'file_id' => 'integer|required',
            'file_id_premio' => 'integer|required',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $insigniaNew = Insignia::create([
                "nombre" => $request->nombre,
                "descripcion_premio" => $request->descripcion_premio,
                "objetivo" => $request->objetivo,
                "user_id" => $request->user_id,
                "file_id" => $request->file_id,
                "file_id_premio" => $request->file_id_premio,
            ]);
            return response()->json([
                'message' => 'La insignia se creó correctamente',
                'data' => $insigniaNew
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear insignia"]]], 400);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Insignia  $insignia
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //Validamos los datos
        $data = $request->only(
            'nombre',
            'descripcion_premio',
            'objetivo',
            'file_id',
            'file_id_premio',
            'estatus'
        );
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion_premio' => 'required|max:600|string',
            'objetivo' => 'required|max:600|string',
            'file_id' => 'integer|required',
            'file_id_premio' => 'integer|required',
            'estatus' => 'required|boolean'
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $insigniaFind = Insignia::findOrfail($id);
            $insigniaFind->update([
                "nombre" => $request->nombre,
                "descripcion_premio" => $request->descripcion_premio,
                "objetivo" => $request->objetivo,
                "file_id" => $request->file_id,
                "file_id_premio" => $request->file_id_premio,
                "estatus" => $request->estatus
            ]);
            return response()->json([
                'message' => 'La insignia se actualizo correctamente',
                'data' => $insigniaFind
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar la insignia"]]], 400);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Insignia  $insignia
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $insigniaFind = Insignia::find($id);
            if (!$insigniaFind)
                return response()->json(['error' => ['mensaje' => ["La insignia no existe"]]], 400);
            $existRelacionesInsignia = $insigniaFind->secureDelete("niveles_retos");
            if ($existRelacionesInsignia)
                return response()->json(['error' => ['mensaje' => ["La insignia ha sido asignada a los niveles retos"]]], 400);
            $insigniaFind->delete();
            return response()->json([
                'message' => 'Se eliminó la insignia correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el insignia"]]], 400);
        }
    }
}
