<?php

namespace App\Http\Controllers\V1\Recompensa;

use App\Http\Controllers\Controller;
use App\Models\HistorialRecompensas;
use App\Models\Participante;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

class RecompensaController extends Controller
{

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $recompensas = HistorialRecompensas::with(['nivel_reto.reto_detail', 'nivel_reto.insignias', 'participante'])
                ->where('estatus', 'pendiente')
                ->get();
            return $recompensas;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('participante_id', 'nivel_id', 'user_id');
        $validator = Validator::make($data, [
            'participante_id' => 'required|integer|exists:participantes,id',
            'nivel_id' => 'required|integer|exists:nivel_retos,id',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');
            $historialNew = HistorialRecompensas::create([
                "fecha_entrega" => $dateNow,
                "participante_id" => $request->participante_id,
                "nivel_id" => $request->nivel_id,
                "user_id" => $request->user_id
            ]);
            return response()->json([
                'message' => 'Se creo la recompensa correctamente',
                'data' => $historialNew
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear la recompensa"]]], 400);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Participante  $participante
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        try {
            //Validamos los datos
            $data = $request->only(
                'user_id',
            );
            $validator = Validator::make($data, [
                'user_id' => 'required|integer|exists:users,id'
            ]);
            //Si falla la validación
            if ($validator->fails()) {
                return response()->json(['error' => $validator->messages()], 400);
            }
            $historialUpdateQuery = DB::transaction(function () use ($request, $id) {
                $currentDateTime = Carbon::now();
                $dateNow = $currentDateTime->format('Y-m-d');
                $historialUpdate = HistorialRecompensas::findOrfail($id);
                $participanteFind = Participante::where('id', $historialUpdate->participante_id)->firstOrFail();
                // $repartidorFind = Participante::where('user_id', $request->user_id)->firstOrFail();

                $historialUpdate->update([
                    "fecha_entrega" => $dateNow,
                    "estatus" => "entregado",
                    "user_id" => $request->user_id
                ]);

                $details = [
                    'tienda' => $participanteFind->nombre_tienda,
                    // 'identificador_repartidor' => $repartidorFind->nombre_completo,
                    'fecha_entrega' => $dateNow,
                    'direccion' => $participanteFind->direccion
                ];
                //se envia el correo
                Mail::to($request->email)->send(new \App\Mail\NotificacionEntregaRecompensa($details));
            });
            return response()->json([
                'message' => 'Se entrego la recompensa correctamente',
                'data' => $historialUpdateQuery
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al entregar la recompensa"]]], 400);
        }
    }
}
